@@ -25,10 +25,15 @@
 #define _COMMON_H
 
 #include <stddef.h>
+#include <glib.h>
 
 unsigned char *ushort2lebytes(unsigned char *buf, unsigned short x);
 unsigned short lebytes2ushort(const unsigned char *buf);
 
 char *hex_dump(const unsigned char *buf, size_t buflen);
 
+#if !GLIB_CHECK_VERSION(2,68,0)
+void* g_memdup2(const void *ptr, size_t size);
+#endif
+
 #endif
