@@ -40,6 +40,7 @@ int vcard_emul_get_login_count(VCard *card);
 /* login into the card, return the 7816 status word (sw2 || sw1) */
 vcard_7816_status_t vcard_emul_login(VCard *card, unsigned char *pin,
                                      int pin_len);
+int vcard_emul_is_logged_in(VCard *card);
 void vcard_emul_logout(VCard *card);
 
 /*
@@ -62,6 +63,7 @@ VCardEmulError vcard_emul_force_card_remove(VReader *vreader);
 
 VCardEmulOptions *vcard_emul_options(const char *args);
 VCardEmulError vcard_emul_init(const VCardEmulOptions *options);
+VCardEmulError vcard_emul_finalize(void);
 void vcard_emul_replay_insertion_events(void);
 void vcard_emul_usage(void);
 
