@@ -6,10 +6,6 @@
 #ifndef VREADERT_H
 #define VREADERT_H 1
 
-#if !defined(__LIBCACARD_H_INSIDE__) && !defined(LIBCACARD_COMPILATION)
-#warning "Only <libcacard.h> can be included directly"
-#endif
-
 typedef enum {
     VREADER_OK = 0,
     VREADER_NO_CARD,
