@@ -22,10 +22,6 @@
 #ifndef VSCARD_COMMON_H
 #define VSCARD_COMMON_H
 
-#if !defined(__LIBCACARD_H_INSIDE__) && !defined(LIBCACARD_COMPILATION)
-#warning "Only <libcacard.h> can be included directly"
-#endif
-
 #include <stdint.h>
 #include <glib.h>
 
