@@ -259,6 +259,33 @@ static void test_sign(void)
     vreader_free(reader); /* get by id ref */
 }
 
+static void test_decipher(void)
+{
+    VReader *reader = vreader_get_reader_by_id(0);
+
+    /* Skip the HW tests without physical card */
+    if (vreader_card_is_present(reader) != VREADER_OK) {
+        vreader_free(reader);
+        g_test_skip("No physical card found");
+        return;
+    }
+
+    /* select the ACA */
+    select_applet(reader, TEST_ACA);
+
+    do_login(reader);
+
+    /* select the PKI */
+    select_applet(reader, TEST_PKI);
+
+    /* get properties to figure out the key length */
+    get_properties(reader, TEST_PKI);
+
+    do_decipher(reader);
+
+    vreader_free(reader); /* get by id ref */
+}
+
 /* Try to pass bad formatted PKCS#1.5 data and make sure the libcacard does not
  * crash while handling them
  */
@@ -269,7 +296,7 @@ static void test_sign_bad_data_x509(void)
     int dwRecvLength = APDUBufSize;
     uint8_t pbRecvBuffer[APDUBufSize];
     uint8_t sign[] = {
-        /* VERIFY   [p1,p2=0 ]  [Lc            ] [2048b keys: 256 bytes of non PKCS#1.5 data] */
+        /* SIGN     [p1,p2=0 ]  [Lc            ] [2048b keys: 256 bytes of non PKCS#1.5 data] */
         0x80, 0x42, 0x00, 0x00, 0x00, 0x01, 0x00,
 0x00, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
 /*      ^--- the second byte of data should be 0x01 for signatures */
@@ -397,6 +424,7 @@ int main(int argc, char *argv[])
     g_test_add_func("/hw-tests/login", test_login);
     g_test_add_func("/hw-tests/sign", test_sign);
     g_test_add_func("/hw-tests/sign-bad-data", test_sign_bad_data_x509);
+    g_test_add_func("/hw-tests/decipher", test_decipher);
     g_test_add_func("/hw-tests/empty-applets", test_empty_applets);
     g_test_add_func("/hw-tests/get-response", test_get_response);
     g_test_add_func("/hw-tests/sign-logout-sign", test_sign_logout_sign);
